@@ -62,10 +62,12 @@ class QueryTaxonTree(query_nosql.DataQuery):
         '''
         names = []
         projection = {'_id': 0, 'tax_name': 1}
-        for _id in ids:
-            query = {'tax_id': _id}
-            cursor = self.collection.find_one(filter=query, projection=projection)
-            names.append(cursor['tax_name'])
+        query = {'tax_id': {'$in': ids}}
+        docs = self.collection.find(filter=query, projection=projection)
+        if docs is None:
+            return ['not in database']
+        for doc in docs:
+            names.append(doc['tax_name'])
         return names
 
     def get_anc_by_name(self, names):
@@ -80,12 +82,12 @@ class QueryTaxonTree(query_nosql.DataQuery):
         result_id = []
         result_name = []
         projection = {'_id': 0, 'anc_id': 1, 'anc_name': 1}
-        for name in names:
-            query = {'tax_name': name}
-            cursor = self.collection.find_one(filter=query, collation=self.collation,
-                                              projection=projection)
-            result_id.append(cursor['anc_id'])
-            result_name.append(cursor['anc_name'])
+        query = {'tax_name': {'$in': names}}
+        docs = self.collection.find(filter=query, collation=self.collation,
+                                    projection=projection)
+        for doc in docs:
+            result_id.append(doc['anc_id'])
+            result_name.append(doc['anc_name'])
         return result_id, result_name
 
     def get_anc_by_id(self, ids):
@@ -100,12 +102,17 @@ class QueryTaxonTree(query_nosql.DataQuery):
         result_name = []
         result_id = []
         projection = {'_id': 0, 'anc_id': 1, 'anc_name': 1}
+        query = {'tax_id': {'$in': ids}}
         for _id in ids:
             query = {'tax_id': _id}
             cursor = self.collection.find_one(query,
                                               projection=projection)
-            result_id.append(cursor['anc_id'])
-            result_name.append(cursor['anc_name'])
+            if cursor is None:
+                result_id.append([-1])
+                result_name.append(['not in database'])
+            else:                
+                result_id.append(cursor['anc_id'])
+                result_name.append(cursor['anc_name'])
         return result_id, result_name
 
     def get_common_ancestor(self, org1, org2, org_format='name'):
@@ -210,28 +217,65 @@ class QueryTaxonTree(query_nosql.DataQuery):
 
         return ids, names
 
-    def get_canon_rank_distance(self, _id):
+    def get_canon_rank_distance(self, _id, front_end=False):
         '''Given the ncbi_id, return canonically-ranked ancestors
             along the lineage and their non-canonical distances
 
             Args:
                 _id (:obj:`int`): ncbi_id of the organism.
+                front_end (:obj:`bool`): meets front_end request
 
             Return:
                 (:obj:`list` of :obj:`dict`): canonical organisms and distances
                 e.g. [{'a':1}, {'b': 3}, ...]
         '''
-        roi = ['species', 'genus', 'family', 'order', 'class', 'phylum', 'kingdom']
+        roi = ['species', 'genus', 'family', 'order', 'class', 'phylum', 'kingdom', 'superkingdom']
         projection = {'rank': 1, '_id': 0, 'tax_name': 1}
         query = {'tax_id': _id}
-        anc = self.collection.find_one(filter=query, projection={'anc_id'})
+        anc = self.collection.find_one(filter=query, projection={'anc_id':1, 'tax_name': 1, 'anc_name': 1})
         result = []
+        if front_end:
+            result.append({anc['tax_name']: 0})
         for i, tax_id in enumerate(reversed(anc['anc_id'])):
             query = {'tax_id': tax_id}
-            doc = self.collection.find_one(filter = query, projection = projection)
+            doc = self.collection.find_one(filter=query, projection=projection)
             rank = doc.get('rank', None)
             if rank in roi:
                 result.append({doc['tax_name']: i + 1})
             else:
                 continue
+        if front_end:
+            result.append({anc['anc_name'][0]: len(anc['anc_name'])})
+        return result
+
+    def get_canon_rank_distance_by_name(self, name, front_end=False):
+        '''Given the name of species, return canonically-ranked ancestors
+            along the lineage and their non-canonical distances
+
+            Args:
+                name (:obj:`str`): name of the organism.
+                front_end (:obj:`bool`): meets front_end request
+
+            Return:
+                (:obj:`list` of :obj:`dict`): canonical organisms and distances
+                e.g. [{'a':1}, {'b': 3}, ...]
+        '''
+        roi = ['species', 'genus', 'family', 'order', 'class', 'phylum', 'kingdom', 'superkingdom']
+        projection = {'rank': 1, '_id': 0, 'tax_name': 1}
+        query = {'tax_name': name}
+        anc = self.collection.find_one(filter=query, projection={'anc_id':1, 'tax_name': 1, 'anc_name': 1},
+                                        collation=self.collation)
+        result = []
+        if front_end:
+            result.append({anc['tax_name']: 0})
+        for i, tax_id in enumerate(reversed(anc['anc_id'])):
+            query = {'tax_id': tax_id}
+            doc = self.collection.find_one(filter=query, projection=projection)
+            rank = doc.get('rank', None)
+            if rank in roi:
+                result.append({doc['tax_name']: i + 1})
+            else:
+                continue
+        if front_end:
+            result.append({anc['anc_name'][0]: len(anc['anc_name'])})
         return result
\ No newline at end of file
